# frozen_string_literal: true

require "is/definable"
require "is/extension"

module Is
  # [public]
  #
  module Factory
    extend Is::Extension

    extends dependencies: [Is::Stateful]

    extends :definition do
      # [public]
      #
      def makes(definable, object, suffix: nil, within: nil)
        unless object.ancestors.include?(Is::Definable)
          object.include(Is::Definable)
        end

        __makes__[definable.to_sym] = {
          object: object,
          suffix: suffix,
          within: Array(within)
        }
      end

      # [public]
      #
      def make(definable, *namespace, name, **kwargs, &block)
        definable = __makes__.fetch(definable.to_sym)

        if (suffix = definable[:suffix])
          name = :"#{name}_#{suffix}"
        end

        definable[:object].define(*definable[:within], *namespace, name, **kwargs, &block)
      end

      # [public]
      #
      def makes?(definable)
        @__makes__.include?(definable.to_sym)
      end

      # [public]
      #
      def makeables
        @__makes__.keys
      end
    end

    applies do
      state :__makes__, :class, default: {}
    end
  end
end
